;
; org-bbdb.el
;
; Author: Sebastian Rose, Hannover, Germany
;
; Released under the GNU General Public License version 3
; see: http://www.gnu.org/licenses/gpl-3.0.html
;

(require 'org)
(require 'bbdb)
(require 'bbdb-com)



(defcustom org-bbdb-default-link-type "bbdb"
  "The default link-type for name completion.
set to 'bbdb', 'mailto' or some custom linktype.")


(defvar org-bbdb-record nil
  "The active bbdb record.")



(define-prefix-command 'org-bbdb-map nil "Org-BBDB")
;; TODO: define these keys in org-mode-map?
(global-set-key [(control ?c) (?b)] org-bbdb-map)
(define-key global-map  [(control ?c) (?b) (?s)]  'org-bbdb/select-record)

(define-key global-map  [(control ?c) (?b) (tab)] 'org-bbdb/complete-name)

(define-key global-map  [(control ?c) (?b) (?n)] 'org-bbdb/insert-name)
(define-key global-map  [(control ?c) (?b) (?m)] 'org-bbdb/insert-mail)
(define-key global-map  [(control ?c) (?b) (?a)] 'org-bbdb/insert-address)
(define-key global-map  [(control ?c) (?b) (?p)] 'org-bbdb/insert-phone)




;; Basic functions

(defun org-bbdb/select-record (&optional cached)
  "Select a new BBDB record to choose entries from.
If CACHED is not nil, use the cached `org-bbdb-record' or select one.
Returns the selected record or nil."
  (interactive)
  (when (or (not cached) (not org-bbdb-record))
    (let ((bbdb-completion-type nil)
          (bbdb-complete-name-allow-cycling t)
          (record (bbdb-completing-read-one-record "Choose record: ")))
      (setq org-bbdb-record record)))
  (unless org-bbdb-record
    (message "%s" "No record selected. Try again."))
  org-bbdb-record)


;; This is the only completion function:

(defun org-bbdb/complete-name ()
  "Complete name at point and insert a link.
NOTE: only the last word preceding point is taken into account for the
search (and thus replaced by the result)!
The type of link depends on the setting of `org-bbdb-default-link-type'.

TODO: select the record."
  (interactive "*")
  (let* ((s (current-word))
         (id (bbdb-read-addresses-with-completion
              "Choose address: " s))
         ;; Not sure if bbdb trims the end of an email.
         ;; Just assume it anyway:
         (has-net (string-match "^\\(.*\\) <\\(.+@.+\\..\\{2,6\\}\\)>$" id))
         (name (if has-net
                   (match-string 1 id) id))
         (mail (if has-net
                   (match-string 2 id) name))
         (lt (if has-net org-bbdb-default-link-type "bbdb"))
         (ins (concat "[[" lt ":" mail "][" name "]]")))

      (backward-word)
      (kill-word 1)
      (insert ins)
      ;; (forward-char (length ins))
      ))


;;
;; Insertion functions
;; We have a get- and a insert- function for all important fields.
;;

;; -- name

(defun org-bbdb/get-name (&optional record)
  "Return the name of a record."
  (let ((rec (or record (org-bbdb/select-record t))))
    (if rec
        (bbdb-record-name rec)
      nil)))

(defun org-bbdb/insert-name (&optional as-link)
  "Insert the name of the currently active record.
With prefix argument, insert a bbdb: Org-link."
  (interactive "*P")
  (let ((n (org-bbdb/get-name)))
    (if n
        (if as-link
            (insert "[[bbdb:" n "][" n "]]")
          (insert n)))))


;; -- email

(defun org-bbdb/get-mail (&optional record)
  "Return the email of a record."
  (let ((rec (or record (org-bbdb/select-record t))))
    (if rec
        (car (bbdb-record-net rec))
      nil)))

(defun org-bbdb/insert-mail (&optional as-link)
  "Insert the email of the currently active record.
With prefix argument, insert a mailto: Org-link."
  (interactive "*P")
  (message "%s" as-link)
  (let ((n (org-bbdb/get-mail)))
    (if n
        (if as-link
            (insert "[[mailto:" n "][" n "]]")
          (insert n)))))


;; -- phone

(defun org-bbdb/get-phone (&optional record)
  "Return the phone number of a record."
  (let ((rec (or record (org-bbdb/select-record t))))
    (if rec
        (car (bbdb-record-phones rec)) ;; support just one phone number...
      nil)))

(defun org-bbdb/insert-phone (&optional as-link)
  "Insert the phone number of the currently active record.
With prefix argument, insert a dial: Org-link which is to be defined."
  (interactive "*P")
  (message "%s" as-link)
  (let ((n (elt (org-bbdb/get-phone) 1)))
    (if n
        (if as-link
            (insert "[[dial:" n "][" n "]]")
          (insert n)))))


;; -- postal address

(defun org-bbdb/get-address (&optional record)
  "Return the postal address of a record."
  (let ((rec (or record (org-bbdb/select-record t))))
    (if rec
        (car (bbdb-record-addresses rec)) ;; we support only one address for now.
      nil)))

(defun org-bbdb/insert-address ()
  "Insert the postal address of the currently active record."
  (interactive "*")
  (let ((addr (org-bbdb/get-address)))
    (if addr
        ;; TODO: add a fomat variable. We will insert the parts of the addresses
        ;; separated by commas in most cases.
        (bbdb-format-address addr nil 0))))
